"""
Scan and count the native docker compose
"""

import json
import socket
import subprocess
from pathlib import Path
from typing import Dict, Optional
from typing import List

import yaml
from pydantic import BaseModel, Field
from rich.console import Console
from rich.progress import track
from rich.table import Table
from tnt.tools.constant import STORAGE_DIR

console = Console()

storage_dir = STORAGE_DIR


class Service(BaseModel):
    # == services configuration == #
    name: str = Field(description="Service name")
    image: str = Field(description="Image name")
    restart: str = Field(default="")
    container_name: str = Field(default="")
    networks: List[str] = Field(default_factory=list)
    env_file: List[str] = Field(default_factory=list)
    ports: List[int] = Field(default_factory=list)
    volumes: List[str] = Field(default_factory=list)
    is_depend_on_gpu: bool = Field(default=False)

    # == running info == #
    container_env: List[str] = Field(
        default_factory=list,
        description="Environment variables from running service via docker compose exec",
    )


class Composer(BaseModel):
    hostname: str = Field(default="default")
    name: str = Field(
        default="",
        description="If not specified, use the absolute path of docker-compose.yaml work_dir",
    )
    status: str
    config_files: Path
    services: List[Service] = Field(description="Services in docker-compose.yaml")


def run_command(command: str, cwd: Optional[str] = None) -> str:
    """Execute command and return output"""
    try:
        result = subprocess.run(command, shell=True, capture_output=True, text=True, cwd=cwd)
        if result.returncode != 0:
            console.print(f"[red]Command execution failed:[/red] {command}")
            console.print(f"[red]Error message:[/red] {result.stderr}")
            return ""
        return result.stdout.strip()
    except Exception as e:
        console.print(f"[red]Error occurred while executing command:[/red] {e}")
        return ""


def get_running_composers() -> List[Dict]:
    """Get all running docker compose projects"""
    cmd = "docker compose ls --format json"
    output = run_command(cmd)
    if not output:
        return []

    try:
        composers = json.loads(output)
        return composers
    except json.JSONDecodeError:
        console.print("[red]Failed to parse docker compose ls output[/red]")
        return []


def get_running_services(project_name: str, work_dir: str) -> List[Dict]:
    """Get running services in specified project"""
    cmd = f"docker compose -p {project_name} ps --format json"
    output = run_command(cmd, cwd=work_dir)
    if not output:
        return []

    try:
        # Output might be multi-line JSON, one service per line
        services = []
        for line in output.strip().split("\n"):
            if line:
                services.append(json.loads(line))
        return services
    except json.JSONDecodeError:
        console.print(f"[red]Failed to parse service list for project {project_name}[/red]")
        return []


def parse_compose_file(file_path: Path) -> Dict:
    """Parse docker-compose.yaml file"""
    try:
        with open(file_path, "r", encoding="utf-8") as f:
            return yaml.safe_load(f)
    except Exception as e:
        console.print(f"[red]Failed to parse compose file {file_path}:[/red] {e}")
        return {}


def get_container_env(container_name: str) -> List[str]:
    """Get container environment variables"""
    cmd = f"docker exec {container_name} env"
    output = run_command(cmd)
    if output:
        return output.strip().split("\n")
    return []


def extract_service_config(service_name: str, service_config: Dict, running_info: Dict) -> Service:
    """Extract service information from configuration"""
    service = Service(
        name=service_name,
        image=service_config.get("image", ""),
        restart=service_config.get("restart", ""),
        container_name=service_config.get("container_name", ""),
        networks=(
            list(service_config.get("networks", {}).keys())
            if isinstance(service_config.get("networks"), dict)
            else service_config.get("networks", [])
        ),
        env_file=(
            service_config.get("env_file", [])
            if isinstance(service_config.get("env_file"), list)
            else [service_config.get("env_file")] if service_config.get("env_file") else []
        ),
        ports=[],
        volumes=service_config.get("volumes", []),
        is_depend_on_gpu=False,
    )

    # Parse port mapping
    ports_config = service_config.get("ports", [])
    for port in ports_config:
        if isinstance(port, str):
            # Format might be "8080:80" or "80"
            port_parts = port.split(":")
            if len(port_parts) >= 2:
                try:
                    service.ports.append(int(port_parts[0]))
                except ValueError:
                    pass
            elif len(port_parts) == 1:
                try:
                    service.ports.append(int(port_parts[0]))
                except ValueError:
                    pass
        elif isinstance(port, dict):
            # New long format
            if "published" in port:
                try:
                    service.ports.append(int(port["published"]))
                except ValueError:
                    pass

    # Check if depends on GPU
    deploy_config = service_config.get("deploy", {})
    if "resources" in deploy_config:
        reservations = deploy_config["resources"].get("reservations", {})
        if "devices" in reservations:
            for device in reservations["devices"]:
                if device.get("driver") == "nvidia":
                    service.is_depend_on_gpu = True
                    break

    # Get container environment variables
    container_name = running_info.get("Name", "")
    if container_name:
        service.container_env = get_container_env(container_name)

    return service


def scan_docker_composes() -> List[Composer]:
    """Scan all running docker compose projects"""
    composers = []
    running_composers = get_running_composers()

    for composer_info in track(
        running_composers, description="Scanning docker compose projects..."
    ):
        project_name = composer_info.get("Name", "")
        config_files = composer_info.get("ConfigFiles", "").split(",")
        status = composer_info.get("Status", "")

        if not project_name or not config_files:
            continue

        # Use the first configuration file
        config_file = Path(config_files[0].strip())
        work_dir = config_file.parent

        # Create Composer object
        composer = Composer(
            hostname=socket.gethostname(),
            name=project_name,
            status=status,
            config_files=config_file,
            services=[],
        )

        # Get running services
        running_services = get_running_services(project_name, str(work_dir))
        if not running_services:
            continue

        # Parse compose file
        compose_config = parse_compose_file(config_file)
        services_config = compose_config.get("services", {})

        # Only process running services
        for running_service in running_services:
            service_name = running_service.get("Service", "")
            if service_name in services_config:
                service = extract_service_config(
                    service_name, services_config[service_name], running_service
                )
                composer.services.append(service)

        composers.append(composer)

    return composers


def save_to_json(composers: List[Composer], output_file: str = "docker_compose_stats.json"):
    """Save results to JSON file"""
    data = []
    for composer in composers:
        # Use mode='json' to automatically handle Path object serialization
        data.append(composer.model_dump(mode="json"))

    storage_path = storage_dir.joinpath(output_file)
    storage_path.parent.mkdir(parents=True, exist_ok=True)
    storage_path.write_text(json.dumps(data, indent=2, ensure_ascii=False), encoding="utf-8")

    console.print(f"[green]Results saved to {output_file}[/green]")


def display_results(composers: List[Composer]):
    """Display scan results in a nice table format"""
    table = Table(title="Docker Compose Projects Scan Results")

    table.add_column("Project Name", style="cyan", no_wrap=True)
    table.add_column("Status", style="magenta")
    table.add_column("Services Count", justify="right", style="green")
    table.add_column("GPU Services", justify="right", style="yellow")
    table.add_column("Total Ports", justify="right", style="blue")

    for composer in composers:
        gpu_services = sum(1 for service in composer.services if service.is_depend_on_gpu)
        total_ports = sum(len(service.ports) for service in composer.services)

        table.add_row(
            composer.name,
            composer.status,
            str(len(composer.services)),
            str(gpu_services),
            str(total_ports),
        )

    console.print(table)


def main(is_to_excel: bool = False, is_visualize: bool = False):
    """Main function"""
    console.print("[bold blue]Starting Docker Compose services scan...[/bold blue]")
    composers = scan_docker_composes()

    if not composers:
        console.print("[yellow]No running Docker Compose projects found[/yellow]")
        return

    console.print(f"[green]Found {len(composers)} running projects[/green]")

    display_results(composers)

    for composer in composers:
        console.print(
            f"  - [cyan]{composer.name}[/cyan]: [green]{len(composer.services)}[/green] services"
        )

    save_to_json(composers, output_file=f"docker_compose_stats_{composers[0].hostname}.json")


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
@Time    : 2025/6/11 09:04
@Author  : QIN2DIM
@GitHub  : https://github.com/QIN2DIM
@Desc    :
"""

import typer
from typer import Option
from rich.console import Console

from tnt.tools import scan_services
from tnt.tools import to_excel
from tnt.tools import visualize

app = typer.Typer(
    name="tnt",
    help="A toolkit for scanning and visualizing docker-compose services.",
    add_completion=False,
)

console = Console()


@app.command()
def main(
    scan: bool = Option(True, "--scan/--no-scan", help="Scan running docker-compose services."),
    excel: bool = Option(True, "--excel/--no-excel", help="Convert scan results to an Excel file."),
    viz: bool = Option(
        True, "--visualize/--no-visualize", help="Visualize service dependencies from scan results."
    ),
):
    """
    TNT command-line interface.
    """
    if not any([scan, excel, viz]):
        console.print("[yellow]No action selected. Use --help for more information.[/yellow]")
        return

    if scan:
        console.print("[bold green]>>> Running Service Scan...[/bold green]")
        try:
            scan_services()
            console.print("[bold green]<<< Service Scan completed.[/bold green]\n")
        except Exception as e:
            console.print(f"[bold red]An error occurred during service scan: {e}[/bold red]")
            # If scan fails, it doesn't make sense to run the other tools
            return

    if excel:
        console.print("[bold green]>>> Generating Excel report...[/bold green]")
        try:
            to_excel()
            console.print("[bold green]<<< Excel report generated.[/bold green]\n")
        except Exception as e:
            console.print(f"[bold red]An error occurred during Excel generation: {e}[/bold red]")

    if viz:
        console.print("[bold green]>>> Generating Service Visualization...[/bold green]")
        try:
            visualize()
            console.print("[bold green]<<< Service Visualization generated.[/bold green]\n")
        except Exception as e:
            console.print(
                f"[bold red]An error occurred during service visualization: {e}[/bold red]"
            )


if __name__ == "__main__":
    app()



"""
简化的服务依赖关系可视化工具
使用Pyvis生成简单的网络图，展示Docker Compose服务和代理依赖
支持将多个主机的数据合并到同一张画布上，展示局域网内的代理共享情况
"""

import json
import os
from collections import defaultdict
from contextlib import suppress
from pathlib import Path
from typing import Dict, List

from pyvis.network import Network
from rich.console import Console
from rich.table import Table

from tnt.tools.constant import STORAGE_DIR, STATS_VIS_FILENAME

console = Console()


class ServiceVisualizer:
    """简化的服务依赖关系可视化器"""

    def __init__(self):
        self.net = None
        self.service_proxy_map = defaultdict(set)  # 服务到代理的映射
        self.proxy_usage_stats = defaultdict(
            lambda: {"hosts": set(), "services": set()}
        )  # 代理使用统计
        self.proxy_positions = {}  # 代理节点的固定位置

    def extract_proxy_info(self, services: List[Dict], hostname: str) -> Dict[str, Dict]:
        """从服务环境变量中提取代理信息

        返回: {proxy_url: {'services': set(), 'details': {...}}}
        """
        proxy_info = defaultdict(lambda: {"services": set(), "details": {}})

        for service in services:
            service_name = service.get("name", "")
            env_vars = service.get("container_env", [])

            for env_var in env_vars:
                if "=" in env_var:
                    key, value = env_var.split("=", 1)
                    # 检查是否是代理相关的环境变量
                    if "proxy" in key.lower() and value.lower().startswith("http://"):
                        proxy_info[value]["services"].add(service_name)
                        proxy_info[value]["details"][key] = value
                        self.service_proxy_map[service_name].add(value)
                        # 记录代理使用统计
                        self.proxy_usage_stats[value]["hosts"].add(hostname)
                        self.proxy_usage_stats[value]["services"].add(f"{hostname}:{service_name}")

        return proxy_info

    def build_network(self, data: List[Dict], is_merged: bool = False):
        """构建简化的网络图

        Args:
            data: 服务数据列表
            is_merged: 是否为合并模式（多个主机）
        """
        # 初始化网络，使用更大的画布和固定布局
        self.net = Network(
            height="1200px", width="100%", bgcolor="#f5f5f5", font_color="#333333", directed=True
        )

        # 使用固定布局，禁用物理引擎
        self.net.set_options(
            """
        var options = {
            "physics": {
                "enabled": false
            },
            "edges": {
                "arrows": {
                    "to": {
                        "enabled": true,
                        "scaleFactor": 0.8
                    }
                },
                "color": {
                    "color": "#666666",
                    "highlight": "#ff0000"
                },
                "smooth": {
                    "type": "curvedCW",
                    "roundness": 0.1
                }
            },
            "nodes": {
                "font": {
                    "size": 14,
                    "face": "Arial"
                }
            },
            "interaction": {
                "hover": true,
                "tooltipDelay": 200
            }
        }
        """
        )

        # 首先收集所有代理信息
        all_proxy_info = defaultdict(lambda: {"services": set(), "details": {}})

        for composer_data in data:
            hostname = composer_data.get("hostname", "unknown")
            services = composer_data.get("services", [])
            proxy_info = self.extract_proxy_info(services, hostname)

            for proxy_url, info in proxy_info.items():
                all_proxy_info[proxy_url]["services"].update(info["services"])
                all_proxy_info[proxy_url]["details"].update(info["details"])

        # 计算布局
        proxy_count = len(all_proxy_info)
        if proxy_count > 0:
            # 代理节点在顶部横向排列
            proxy_spacing = 1200 / (proxy_count + 1)
            for i, proxy_url in enumerate(sorted(all_proxy_info.keys())):
                x = (i + 1) * proxy_spacing - 600
                y = -400  # 顶部位置
                self.proxy_positions[proxy_url] = (x, y)

        # 添加代理节点（置顶）
        self._add_proxy_nodes_pinned(all_proxy_info, is_merged)

        # 添加主机、服务等节点
        host_count = len(set(d.get("hostname", "unknown") for d in data))
        host_spacing = 1200 / (host_count + 1) if host_count > 0 else 600
        host_index = 0

        for composer_data in data:
            hostname = composer_data.get("hostname", "unknown")

            # 计算主机位置
            host_x = (host_index + 1) * host_spacing - 600
            host_y = 0

            # 添加主机节点
            self.net.add_node(
                hostname,
                label=hostname,
                color="#2196F3",
                x=host_x,
                y=host_y,
                size=30,
                shape="box",
                font={"size": 16, "color": "white"},
                title=f"主机: {hostname}",
            )

            # 添加Compose项目和服务
            composer_name = composer_data.get("name", "")
            composer_id = f"{hostname}_{composer_name}"

            # Compose项目位置
            self.net.add_node(
                composer_id,
                label=composer_name,
                color="#4CAF50",
                x=host_x,
                y=host_y + 150,
                size=25,
                shape="box",
                title=f"Docker Compose: {composer_name}",
            )

            # 连接主机到Compose项目
            self.net.add_edge(hostname, composer_id, color="#cccccc", width=1)

            # 添加服务节点
            services = composer_data.get("services", [])
            service_count = len(services)
            if service_count > 0:
                service_spacing = 200 / (service_count + 1) if service_count > 1 else 0

                for j, service in enumerate(services):
                    service_name = service.get("name", "")
                    service_id = f"{composer_id}_{service_name}"

                    # 计算服务位置
                    service_x = host_x + (j - service_count / 2 + 0.5) * service_spacing
                    service_y = host_y + 300

                    # 服务颜色
                    color = "#FFA726" if service.get("is_depend_on_gpu") else "#66BB6A"

                    self.net.add_node(
                        service_id,
                        label=service_name,
                        color=color,
                        x=service_x,
                        y=service_y,
                        size=20,
                        shape="dot",
                        title=f"服务: {service_name}\n镜像: {service.get('image', 'N/A')}",
                    )

                    # 连接Compose项目到服务
                    self.net.add_edge(composer_id, service_id, color="#cccccc", width=1)

                    # 连接服务到代理
                    self._connect_service_to_proxies(
                        service_id, service_name, hostname, all_proxy_info
                    )

            host_index += 1

    def _add_proxy_nodes_pinned(self, proxy_info: Dict[str, Dict], is_merged: bool = False):
        """添加固定位置的代理节点"""
        from collections import defaultdict

        for proxy_url, (x, y) in self.proxy_positions.items():
            info = proxy_info[proxy_url]
            proxy_id = f"proxy_{proxy_url}"
            display_name = proxy_url.replace("http://", "").split(":")[0]

            # 获取使用统计
            usage_stats = self.proxy_usage_stats.get(proxy_url, {"hosts": set(), "services": set()})

            # 简洁的标签
            is_shared = is_merged and len(usage_stats["hosts"]) > 1
            label = f"{display_name}\n({len(usage_stats['services'])} 服务)"

            # 构建详细的提示信息 - 包含完整的服务依赖列表
            proxy_details = []
            proxy_details.append(f"🔗 代理服务器: {proxy_url}")
            proxy_details.append(f"━━━━━━━━━━━━━━━━━━━━━━━━━━━━")
            proxy_details.append(f"📊 使用统计:")
            proxy_details.append(f"  • 主机数量: {len(usage_stats['hosts'])}")
            proxy_details.append(f"  • 服务总数: {len(usage_stats['services'])}")
            proxy_details.append(f"  • 跨主机共享: {'是' if is_shared else '否'}")

            proxy_details.append(f"\n📋 使用此代理的服务详情:")

            # 按主机分组显示服务
            host_services = defaultdict(list)
            for service_full in sorted(usage_stats["services"]):
                if ":" in service_full:
                    host, service = service_full.split(":", 1)
                    host_services[host].append(service)

            for host in sorted(host_services.keys()):
                proxy_details.append(f"\n  🖥️ {host}:")
                for service in sorted(host_services[host]):
                    proxy_details.append(f"    • {service}")

            # 添加环境变量配置
            if info["details"]:
                proxy_details.append(f"\n⚙️ 环境变量配置:")
                for env_key in sorted(info["details"].keys()):
                    proxy_details.append(f"  • {env_key}")

            title = "\n".join(proxy_details)

            # 添加代理节点 - 使用醒目的颜色
            self.net.add_node(
                proxy_id,
                label=label,
                color="#E91E63" if is_shared else "#9C27B0",
                x=x,
                y=y,
                size=40,
                shape="star",
                font={"size": 16, "color": "white", "bold": True},
                title=title,
                physics=False,  # 确保位置固定
            )

    def _connect_service_to_proxies(
        self, service_id: str, service_name: str, hostname: str, proxy_info: Dict[str, Dict]
    ):
        """连接服务到其使用的代理"""
        service_key = f"{hostname}:{service_name}"

        for proxy_url, info in proxy_info.items():
            usage_stats = self.proxy_usage_stats.get(proxy_url, {"hosts": set(), "services": set()})
            if service_key in usage_stats["services"]:
                proxy_id = f"proxy_{proxy_url}"
                # 使用醒目的红色连线表示代理依赖
                self.net.add_edge(
                    service_id,
                    proxy_id,
                    color="#F44336",
                    width=2,
                    dashes=False,
                    title=f"使用代理: {proxy_url}",
                )

    def generate_summary_table(self) -> Table:
        """生成代理使用摘要表格"""
        table = Table(title="代理使用摘要", show_header=True, header_style="bold magenta")
        table.add_column("代理地址", style="cyan", no_wrap=True)
        table.add_column("主机数", justify="center", style="green")
        table.add_column("服务数", justify="center", style="green")
        table.add_column("跨主机共享", justify="center", style="yellow")
        table.add_column("使用的主机", style="dim")

        for proxy_url, stats in sorted(self.proxy_usage_stats.items()):
            is_shared = len(stats["hosts"]) > 1
            hosts_str = ", ".join(sorted(stats["hosts"])[:3])
            if len(stats["hosts"]) > 3:
                hosts_str += f" ... (+{len(stats['hosts']) - 3})"

            table.add_row(
                proxy_url,
                str(len(stats["hosts"])),
                str(len(stats["services"])),
                "✓" if is_shared else "✗",
                hosts_str,
            )

        return table

    def save_visualization(self, output_file: str = "service_network.html"):
        """保存可视化结果，并添加自定义样式"""
        if self.net:
            # 先生成基础HTML
            self.net.save_graph(output_file)

            # 读取并修改HTML，添加自定义样式和说明
            with open(output_file, "r", encoding="utf-8") as f:
                html_content = f.read()

            # 生成统计信息HTML
            stats_html = self._generate_stats_html()

            # 添加自定义样式和说明面板
            custom_style = """
            <style>
                body { margin: 0; padding: 0; font-family: Arial, sans-serif; }
                #mynetwork { height: 100vh; }
                .info-panel {
                    position: absolute;
                    top: 10px;
                    right: 10px;
                    background: white;
                    border: 1px solid #ddd;
                    border-radius: 5px;
                    padding: 15px;
                    box-shadow: 0 2px 4px rgba(0,0,0,0.1);
                    font-size: 14px;
                    max-width: 350px;
                    max-height: 90vh;
                    overflow-y: auto;
                }
                .info-panel h3 { margin-top: 0; color: #333; }
                .legend-item { margin: 5px 0; }
                .legend-color {
                    display: inline-block;
                    width: 20px;
                    height: 20px;
                    margin-right: 8px;
                    vertical-align: middle;
                    border-radius: 3px;
                }
                .stats-section {
                    margin-top: 15px;
                    padding-top: 15px;
                    border-top: 1px solid #eee;
                }
                .stats-table {
                    width: 100%;
                    font-size: 12px;
                    border-collapse: collapse;
                    margin-top: 10px;
                }
                .stats-table th {
                    background: #f0f0f0;
                    padding: 5px;
                    text-align: left;
                    font-weight: bold;
                    border-bottom: 1px solid #ddd;
                }
                .stats-table td {
                    padding: 5px;
                    border-bottom: 1px solid #eee;
                }
                .shared-proxy {
                    color: #E91E63;
                    font-weight: bold;
                }
                .proxy-details {
                    font-size: 11px;
                    color: #666;
                    margin-left: 10px;
                }
            </style>
            """

            info_panel = f"""
            <div class="info-panel">
                <h3>🎯 代理网络拓扑图</h3>
                <div class="legend-item">
                    <span class="legend-color" style="background: #E91E63;"></span>
                    <span>共享代理（多主机）</span>
                </div>
                <div class="legend-item">
                    <span class="legend-color" style="background: #9C27B0;"></span>
                    <span>独占代理（单主机）</span>
                </div>
                <div class="legend-item">
                    <span class="legend-color" style="background: #2196F3;"></span>
                    <span>主机</span>
                </div>
                <div class="legend-item">
                    <span class="legend-color" style="background: #4CAF50;"></span>
                    <span>Docker Compose</span>
                </div>
                <div class="legend-item">
                    <span class="legend-color" style="background: #FFA726;"></span>
                    <span>GPU服务</span>
                </div>
                <div class="legend-item">
                    <span class="legend-color" style="background: #66BB6A;"></span>
                    <span>普通服务</span>
                </div>
                
                {stats_html}
                
                <hr style="margin: 15px 0;">
                <p style="margin: 5px 0; color: #666; font-size: 12px;">
                    <strong>💡 使用提示：</strong><br>
                    • 代理节点固定在顶部<br>
                    • 红色连线表示代理依赖<br>
                    • 鼠标悬停查看详细信息<br>
                    • 可拖拽和缩放视图
                </p>
            </div>
            """

            # 在</head>标签前插入自定义样式
            html_content = html_content.replace("</head>", custom_style + "</head>")

            # 在<body>标签后插入信息面板
            html_content = html_content.replace("<body>", "<body>" + info_panel)

            # 写回文件
            with open(output_file, "w", encoding="utf-8") as f:
                f.write(html_content)

            console.print(f"[green]✅ 可视化结果已保存到: {output_file}[/green]")
        else:
            console.print("[red]❌ 网络图未构建[/red]")

    def _generate_stats_html(self) -> str:
        """生成统计信息的HTML内容"""
        if not self.proxy_usage_stats:
            return ""

        # 计算统计数据
        total_proxies = len(self.proxy_usage_stats)
        shared_proxies = sum(
            1 for stats in self.proxy_usage_stats.values() if len(stats["hosts"]) > 1
        )
        total_services = sum(len(stats["services"]) for stats in self.proxy_usage_stats.values())
        unique_hosts = set()
        for stats in self.proxy_usage_stats.values():
            unique_hosts.update(stats["hosts"])

        stats_html = f"""
        <div class="stats-section">
            <h4>📊 统计信息</h4>
            <div style="display: grid; grid-template-columns: 1fr 1fr; gap: 10px; margin-bottom: 10px;">
                <div>
                    <strong>代理总数:</strong> {total_proxies}<br>
                    <strong>共享代理:</strong> <span class="shared-proxy">{shared_proxies}</span>
                </div>
                <div>
                    <strong>主机总数:</strong> {len(unique_hosts)}<br>
                    <strong>服务总数:</strong> {total_services}
                </div>
            </div>
            
            <h4>🔗 代理使用详情</h4>
            <table class="stats-table">
                <thead>
                    <tr>
                        <th>代理</th>
                        <th>主机</th>
                        <th>服务</th>
                    </tr>
                </thead>
                <tbody>
        """

        # 按共享程度排序（共享的代理排在前面）
        sorted_proxies = sorted(
            self.proxy_usage_stats.items(),
            key=lambda x: (len(x[1]["hosts"]), len(x[1]["services"])),
            reverse=True,
        )

        for proxy_url, stats in sorted_proxies:
            is_shared = len(stats["hosts"]) > 1
            proxy_display = proxy_url.replace("http://", "").split(":")[0]

            # 获取服务列表
            host_services = defaultdict(list)
            for service_full in stats["services"]:
                if ":" in service_full:
                    host, service = service_full.split(":", 1)
                    host_services[host].append(service)

            # 生成服务详情
            services_detail = []
            for host in sorted(host_services.keys()):
                services_detail.append(f"{host}: {', '.join(sorted(host_services[host]))}")

            row_class = "shared-proxy" if is_shared else ""
            stats_html += f"""
                <tr>
                    <td class="{row_class}">{proxy_display}</td>
                    <td>{len(stats['hosts'])}</td>
                    <td>{len(stats['services'])}</td>
                </tr>
            """

        stats_html += """
                </tbody>
            </table>
        </div>
        """

        return stats_html


def load_json_files(path: Path) -> List[Dict]:
    """加载单个JSON文件或目录下的所有JSON文件"""
    all_data = []

    if path.is_file():
        try:
            with open(path, "r", encoding="utf-8") as f:
                data = json.load(f)
                if isinstance(data, list):
                    all_data.extend(data)
                else:
                    all_data.append(data)
        except Exception as e:
            console.print(f"[red]❌ 无法读取JSON文件 {path}: {e}[/red]")

    elif path.is_dir():
        json_files = list(path.glob("*.json"))
        if not json_files:
            console.print(f"[yellow]⚠️ 目录 {path} 中没有找到JSON文件[/yellow]")
            return all_data

        console.print(f"[cyan]📂 找到 {len(json_files)} 个JSON文件[/cyan]")
        for json_file in json_files:
            try:
                with open(json_file, "r", encoding="utf-8") as f:
                    data = json.load(f)
                    if isinstance(data, list):
                        all_data.extend(data)
                    else:
                        all_data.append(data)
                console.print(f"  ✓ 加载 {json_file.name}")
            except Exception as e:
                console.print(f"  [red]✗ 无法读取 {json_file.name}: {e}[/red]")

    return all_data


def main(
    path_obj: Path = STORAGE_DIR, merge: bool = True, merged_filename: str = STATS_VIS_FILENAME
):
    """Docker Compose服务依赖关系可视化

    PATH: 输入的JSON文件或包含JSON文件的目录路径

    当输入为目录时，默认会将所有JSON文件合并到同一张画布上，
    以展示整个局域网内的代理共享情况。
    """
    console.print(f"[bold blue]🔍 开始分析服务依赖关系...[/bold blue]")

    # 加载数据
    all_data = load_json_files(path_obj)

    if not all_data:
        console.print("[red]❌ 没有有效的数据可以处理[/red]")
        return

    # 根据是否为目录和merge选项决定处理方式
    if path_obj.is_dir() and merge:
        # 合并模式：所有数据在同一张画布上
        console.print(f"[cyan]🔀 合并模式：将 {len(all_data)} 个数据源合并到同一张画布[/cyan]")

        visualizer = ServiceVisualizer()
        visualizer.build_network(all_data, is_merged=True)

        # 统计信息
        total_hosts = len(set(d.get("hostname", "unknown") for d in all_data))
        total_composers = len(all_data)
        total_services = sum(len(composer.get("services", [])) for composer in all_data)
        shared_proxies = sum(
            1 for stats in visualizer.proxy_usage_stats.values() if len(stats["hosts"]) > 1
        )

        console.print(f"\n[green]📊 统计信息:[/green]")
        console.print(f"  - 主机数: {total_hosts}")
        console.print(f"  - Compose项目数: {total_composers}")
        console.print(f"  - 服务总数: {total_services}")
        console.print(f"  - 代理总数: {len(visualizer.proxy_usage_stats)}")
        console.print(f"  - 共享代理数: {shared_proxies}")

        # 显示代理使用摘要表格
        if visualizer.proxy_usage_stats:
            console.print("\n")
            console.print(visualizer.generate_summary_table())

        output_file = str(path_obj.joinpath(merged_filename))
        visualizer.save_visualization(output_file)

        # 给出更清晰的指引
        console.print(f"\n[bold yellow]📌 重要提示:[/bold yellow]")
        console.print(f"  • 所有代理节点已固定在顶部，便于查看")
        console.print(f"  • 红色连线表示服务对代理的依赖关系")
        console.print(f"  • 共享代理使用星形图标和醒目颜色标注")
        console.print(f"  • 鼠标悬停在节点上可查看详细信息")
        with suppress(Exception):
            os.startfile(output_file)
    else:
        # 分离模式：每个数据源生成独立的画布
        console.print(f"[cyan]📄 分离模式：为每个数据源生成独立的画布[/cyan]")

        for i, composer_data in enumerate(all_data):
            visualizer = ServiceVisualizer()
            visualizer.build_network([composer_data])

            hostname = composer_data.get("hostname", f"unknown_{i}")
            composer_name = composer_data.get("name", f"composer_{i}")
            output_file = str(
                path_obj.parent.joinpath(f"service_network_{hostname}_{composer_name}.html")
            )

            visualizer.save_visualization(output_file)

    console.print(f"\n[bold green]✨ 可视化完成！[/bold green]")


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
@Time    : 2025/6/11 09:11
@Author  : QIN2DIM
@GitHub  : https://github.com/QIN2DIM
@Desc    :
"""
from .scan_services import main as scan_services
from .json_to_excel import main as to_excel
from .visualize_services import main as visualize



# -*- coding: utf-8 -*-
"""
@Time    : 2025/6/11 09:23
@Author  : QIN2DIM
@GitHub  : https://github.com/QIN2DIM
@Desc    :
"""
from pathlib import Path

STORAGE_DIR = Path("stats")
STATS_EXCEL_FILENAME = "docker_compose_all_services.xlsx"
STATS_VIS_FILENAME = "merged_service_network.html"



"""
数据转换器：将docker compose扫描结果JSON转换为Excel表格
"""

import json
import os
from contextlib import suppress
from pathlib import Path
from typing import List, Dict, Any

import pandas as pd
from openpyxl.styles import Font

from tnt.tools.constant import STORAGE_DIR, STATS_EXCEL_FILENAME


def convert_list_to_str(value: Any) -> str:
    """将列表类型数据转换为字符串"""
    if isinstance(value, list):
        return "; ".join(str(item) for item in value)
    return str(value) if value is not None else ""


def load_json_data(json_file: str) -> List[Dict]:
    """加载JSON数据"""
    file_path = Path(json_file)
    if not file_path.exists():
        raise FileNotFoundError(f"JSON文件不存在: {json_file}")

    with open(file_path, "r", encoding="utf-8") as f:
        return json.load(f)


def extract_services_data(composers_data: List[Dict]) -> List[Dict]:
    """提取所有services数据"""
    services_list = []

    for composer in composers_data:
        hostname = composer.get("hostname", "")
        project_name = composer.get("name", "")
        project_status = composer.get("status", "")
        config_files = composer.get("config_files", "")

        for service in composer.get("services", []):
            service_data = {
                "项目主机名": hostname,
                "项目名称": project_name,
                "项目状态": project_status,
                "配置文件": config_files,
                "服务名称": service.get("name", ""),
                "镜像": service.get("image", ""),
                "重启策略": service.get("restart", ""),
                "容器名称": service.get("container_name", ""),
                "网络": convert_list_to_str(service.get("networks", [])),
                "环境文件": convert_list_to_str(service.get("env_file", [])),
                "端口": convert_list_to_str(service.get("ports", [])),
                "挂载卷": convert_list_to_str(service.get("volumes", [])),
                "依赖GPU": service.get("is_depend_on_gpu", False),
                "容器环境变量": convert_list_to_str(service.get("container_env", [])),
            }
            services_list.append(service_data)

    return services_list


def convert_json_to_excel(json_file: str, output_file: str = ""):
    """转换JSON到Excel"""
    # 加载数据
    composers_data = load_json_data(json_file)

    # 提取services数据
    services_data = extract_services_data(composers_data)

    if not services_data:
        print("没有找到services数据")
        return

    # 创建DataFrame
    df = pd.DataFrame(services_data)

    # 生成输出文件名
    if not output_file:
        json_path = Path(json_file)
        output_file = json_path.parent / f"{json_path.stem}_services.xlsx"

    # 保存到Excel
    with pd.ExcelWriter(output_file, engine="openpyxl") as writer:
        df.to_excel(writer, sheet_name="Services", index=False)

        # 设置字体为微软雅黑
        worksheet = writer.sheets["Services"]
        font = Font(name="微软雅黑", size=10)

        for row in worksheet.iter_rows():
            for cell in row:
                cell.font = font

    print(f"数据已转换为Excel文件: {output_file}")
    print(f"共导出 {len(services_data)} 个服务")


def convert_multiple_json_to_excel(json_files: List[str], output_file: str):
    """将多个JSON文件转换为一个Excel文件的多个sheet"""
    if not json_files:
        print("没有找到JSON文件")
        return

    total_services = 0

    # 创建Excel写入器
    with pd.ExcelWriter(output_file, engine="openpyxl") as writer:
        for json_file in json_files:
            try:
                # 加载数据
                composers_data = load_json_data(json_file)

                # 提取services数据
                services_data = extract_services_data(composers_data)

                if not services_data:
                    print(f"在 {json_file} 中没有找到services数据")
                    continue

                # 获取hostname作为sheet名称
                hostname = (
                    composers_data[0].get("hostname", "Unknown") if composers_data else "Unknown"
                )
                # 清理sheet名称（Excel sheet名称有特殊字符限制）
                sheet_name = (
                    hostname.replace("/", "_").replace("\\", "_").replace(":", "_")[:31]
                )  # Excel sheet名称最大31个字符

                # 创建DataFrame
                df = pd.DataFrame(services_data)

                # 写入到对应的sheet
                df.to_excel(writer, sheet_name=sheet_name, index=False)

                # 自适应列宽和设置字体
                worksheet = writer.sheets[sheet_name]

                # 设置字体为微软雅黑
                font = Font(name="微软雅黑", size=10)

                for column in worksheet.columns:
                    max_length = 0
                    column_letter = column[0].column_letter

                    for cell in column:
                        # 设置字体
                        cell.font = font

                        try:
                            if len(str(cell.value)) > max_length:
                                max_length = len(str(cell.value))
                        except:
                            pass

                    # 设置列宽，最小宽度10，最大宽度50
                    adjusted_width = min(max(max_length + 2, 10), 50)
                    worksheet.column_dimensions[column_letter].width = adjusted_width

                total_services += len(services_data)
                print(f"已处理 {json_file}，主机名: {hostname}，服务数: {len(services_data)}")

            except Exception as e:
                print(f"处理文件 {json_file} 时出错: {e}")
                continue

    print(f"数据已转换为Excel文件: {output_file}")
    print(f"共导出 {total_services} 个服务")


def main(storage_dir: Path = STORAGE_DIR, filename: str = STATS_EXCEL_FILENAME):
    """主函数"""
    # 查找storage目录下的JSON文件
    if not storage_dir.exists():
        return

    json_files = list(storage_dir.glob("*.json"))
    if not json_files:
        print("在storage目录下没有找到JSON文件")
        return

    # 生成输出文件名
    output_file = storage_dir / filename

    # 转换为Excel
    print(f"找到 {len(json_files)} 个JSON文件")
    convert_multiple_json_to_excel([str(jf) for jf in json_files], str(output_file))

    with suppress(Exception):
        os.startfile(output_file)


if __name__ == "__main__":
    main()
